#define F_CPU 16000000UL

#include <avr/io.h>
#include <util/delay.h>

#include "font.h"

extern unsigned int X_SIZE;
extern unsigned int Y_SIZE;
extern unsigned long XY_LEN;

//----------------------  
#define DATA_HI_DDR   	DDRL
#define DATA_HI_PORT  	PORTL
//----------------------
#define RST_DDR  	DDRD
#define RST_PORT 	PORTD
#define RST			7

#define LCD_C (*(volatile uint8_t*)(0x8000))
#define LCD_D (*(volatile uint8_t*)(0xC000))

extern unsigned int ccc[256];

//==============================================================================
void sendCom(unsigned char cmd)
{	LCD_C = cmd;
}
//==============================================================================
void sendData(unsigned char data)
{	LCD_D = data;
}
//==============================================================================
void sendData16(unsigned int data)
{	DATA_HI_PORT = (data>>8);
	LCD_D = data;
}
//==============================================================================
void setAddrWindow(
	unsigned int x1,unsigned int y1,
	unsigned int x2,unsigned int y2)
{	sendCom(0x2A);
	sendData(x1>>8);	sendData(x1);
	sendData(x2>>8);	sendData(x2);

	sendCom(0x2B);
	sendData(y1>>8);	sendData(y1);
	sendData(y2>>8);	sendData(y2);
}
//==============================================================================
void drawPixel(unsigned int x, unsigned int y, unsigned int color)
{  	setAddrWindow(x,y,x,y);//X_SIZE-1,Y_SIZE-1);
	sendCom(0x2C);
  	sendData16(color);
}
//==============================================================================
void drawChar(
	unsigned int x, unsigned int y,
	unsigned int color, unsigned int backcolor, 
	unsigned char ch)
{	
	unsigned char value = 0;
	unsigned char mask[8] ={128,64,32,16,8,4,2,1};

   	int v,h;
	int tcolor;

	for(h = 0; h < 5; h++)
  	{	
		value = pgm_read_byte(&(font5x8[ch-32][h]));

		for(v = 0; v < 8; v++)
		{	if ((value & mask[v]) != 0) tcolor = color;
			else tcolor = backcolor;
			drawPixel(x+v,y+h,tcolor);
		}
  	}
}
//==============================================================================
void print_str(
		unsigned char x_pos, unsigned char y_pos,
		unsigned int colour, unsigned int back_colour,
		char *str)
{	while (*str)
	{	drawChar(x_pos,y_pos,colour,back_colour,*str++);
		y_pos += 6;
	}
}
//==============================================================================
//   
// TCNT1 = 507 = 32448 u
void fillArea(unsigned int color, unsigned long len)
{  	unsigned int blocks = (unsigned short)(len/16);
  	unsigned char hi = (color>>8), lo = color;

	LCD_C = 0x2C;
	LCD_D = hi; LCD_D = lo;

	DATA_HI_PORT = hi;
	while(blocks--)
	{
		LCD_D = lo; LCD_D = lo; LCD_D = lo; LCD_D = lo; 
		LCD_D = lo; LCD_D = lo; LCD_D = lo; LCD_D = lo; 
		LCD_D = lo; LCD_D = lo; LCD_D = lo; LCD_D = lo; 
		LCD_D = lo; LCD_D = lo; LCD_D = lo; LCD_D = lo; 
	}
}
//==============================================================================
//   
// TCNT1 = 2466 = 157824 u
void fillAreaM(unsigned int color, unsigned long len)
{  	unsigned int blocks = (unsigned int)(len/16);
  	unsigned char i = 0, hi = (color>>8), lo = color;

	LCD_C = 0x2C;
	LCD_D = hi; LCD_D = lo;
 
#define mFOUT \
	DATA_HI_PORT = (ccc[i]>>8);	LCD_D = ccc[i];\
	i++; i &= 255; 

	while(blocks--)
	{
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			

}
//==============================================================================
void fillScreen(unsigned int color)
{ 	setAddrWindow(0,0,X_SIZE-1,Y_SIZE-1);
	fillArea(color,XY_LEN);//(long)X_SIZE*(long)Y_SIZE);
}
//==============================================================================
void fillScreenM(unsigned int color)
{ 	setAddrWindow(0,0,X_SIZE-1,Y_SIZE-1);
	fillAreaM(color,XY_LEN);//(long)X_SIZE*(long)Y_SIZE);
}
//==============================================================================
//==============================================================================
void init9481(void)
{
	sendCom(0x11);

	_delay_ms(20);

	sendCom(0xD0);
	sendData(0x07);
	sendData(0x42);
	sendData(0x18);

	sendCom(0xD1);
	sendData(0x00);
	sendData(0x07);
	sendData(0x10);

	sendCom(0xD2);
	sendData(0x01);
	sendData(0x02);

	sendCom(0xC0);
	sendData(0x10);
	sendData(0x3B);
	sendData(0x00);
	sendData(0x02);
	sendData(0x11);

	sendCom(0xC5);
	sendData(0x03);

	sendCom(0xC8);
	sendData(0x00);
	sendData(0x32);
	sendData(0x36);
	sendData(0x45);
	sendData(0x06);
	sendData(0x16);
	sendData(0x37);
	sendData(0x75);
	sendData(0x77);
	sendData(0x54);
	sendData(0x0C);
	sendData(0x00);

 	sendCom(0x36);
	sendData(0xCA);
	X_SIZE = 320;	Y_SIZE = 480;

	sendCom(0x3A);
	sendData(0x55);

	sendCom(0x2A);
	sendData(0x00);
	sendData(0x00);
	sendData(0x01);
	sendData(0x3F);

	sendCom(0x2B);
	sendData(0x00);
	sendData(0x00);
	sendData(0x01);
	sendData(0xE0);
	_delay_ms(120);
	sendCom(0x29);
}
//==============================================================================
void initTFT(void)
{
	DATA_HI_DDR = 0xFF;

//-----------------------   
	RST_DDR  |= (1<<RST);
	RST_PORT &= ~(1<<RST);
	_delay_ms(10);
	RST_PORT |= (1<<RST);
	_delay_ms(120);

	init9481();
}
//==============================================================================
