#define F_CPU 16000000UL

#include <avr/io.h>
#include <util/delay.h>

#define DATA_LO_DDR   	DDRA
#define DATA_LO_PORT  	PORTA
//----------------------  
#define DATA_HI_DDR   	DDRL
#define DATA_HI_PORT  	PORTL

//----------------------
#define WR_DDR   	DDRG
#define WR_PORT  	PORTG

#define WR_DDR   	DDRG
#define WR_PORT  	PORTG
#define WR   	 	0
#define RD   	 	1
//----------------------
#define RS_DDR			DDRC
#define RS_PORT  		PORTC
#define RS   	 		6 
//----------------------
#define RST_DDR  		DDRD
#define RST_PORT 		PORTD
#define RST			  	7
//----------------------

#include "font.h"

extern unsigned int X_SIZE;
extern unsigned int Y_SIZE;
extern unsigned long XY_LEN;

extern unsigned int ccc[256];

//==============================================================================
void sendCom(unsigned char cmd)
{  	RS_PORT &= ~(1<<RS);
	DATA_LO_PORT = cmd;

	WR_PORT &= ~(1<<WR);
	WR_PORT |= (1<<WR);
}
//==============================================================================
void sendData(unsigned char data)
{  	RS_PORT |= (1<<RS);
	DATA_LO_PORT = data;

	WR_PORT &= ~(1<<WR);
	WR_PORT |= (1<<WR);
}
//==============================================================================
void sendData16(unsigned int data)
{  	RS_PORT |= (1<<RS);
	
	DATA_HI_PORT = (data>>8);
	DATA_LO_PORT = data;

	WR_PORT &= ~(1<<WR);
//	mDELAY
	WR_PORT |= (1<<WR);
}
//==============================================================================
void setAddrWindow(
	unsigned int x1,unsigned int y1,
	unsigned int x2,unsigned int y2)
{	sendCom(0x2A);
	sendData(x1>>8);	sendData(x1);
	sendData(x2>>8);	sendData(x2);

	sendCom(0x2B);
	sendData(y1>>8);	sendData(y1);
	sendData(y2>>8);	sendData(y2);
}
//==============================================================================
void drawPixel(unsigned int x, unsigned int y, unsigned int color)
{
  	setAddrWindow(x,y,x,y);
	sendCom(0x2C);
  	sendData16(color);
}
//==============================================================================
void drawChar(
	unsigned int x, unsigned int y,
	unsigned int color, unsigned int backcolor, 
	unsigned char ch)
{	
	unsigned char value = 0;
	unsigned char mask[8] ={128,64,32,16,8,4,2,1};

   	int v,h;
	int tcolor;

	for(h = 0; h < 5; h++)
  	{	
		value = pgm_read_byte(&(font5x8[ch-32][h]));

		for(v = 0; v < 8; v++)
		{	if ((value & mask[v]) != 0) tcolor = color;
			else tcolor = backcolor;
			drawPixel(x+v,y+h,tcolor);
		}
  	}
}
//==============================================================================
void print_str(
	unsigned int x_pos, unsigned int y_pos,
	unsigned int color, unsigned int back_color,
	char *str)
{	while (*str)
	{	drawChar(x_pos,y_pos,color,back_color,*str++);
		y_pos += 6;
	}
}
//==============================================================================
//   
// TCNT1=656 = 41984 uSec
void fillArea(unsigned int color, unsigned long len)
{
  	unsigned int blocks;
	sendCom(0x2C);
	sendData16(color);

	len--;

	DATA_HI_PORT = (color>>8);
	DATA_LO_PORT = color;

	blocks = (unsigned int)(len/16);
 
//  
#define mFOUT WR_PORT &= ~(1<<WR);WR_PORT |= (1<<WR);

	while(blocks--)
	{	
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			
}
//==============================================================================
//   
// TCNT1=2916 = 186624 usec
void fillAreaM(unsigned int color, unsigned long len)
{
  	unsigned int blocks = (unsigned int)(len/16);
	unsigned int i = 0;

	sendCom(0x2C);
	sendData16(color);

//	DATA_HI_PORT = hi;	DATA_LO_PORT = color;

#define mFOUT \
	DATA_HI_PORT = (ccc[i]>>8);	DATA_LO_PORT = ccc[i];\
	WR_PORT &= ~(1<<WR);WR_PORT |= (1<<WR);\
	i++; i &= 255; 

	while(blocks--)
	{
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			

}
//==============================================================================
void fillScreen(unsigned int color)
{ 	setAddrWindow(0,0,X_SIZE-1,Y_SIZE-1);
  	fillArea(color,XY_LEN);
}
//==============================================================================
void fillScreenM(unsigned int color)
{ 	setAddrWindow(0,0,X_SIZE-1,Y_SIZE-1);
 	fillAreaM(color, XY_LEN);
}
//==============================================================================
//==============================================================================
void init9481(void)
{
	sendCom(0x11);

	_delay_ms(20);

	sendCom(0xD0);
	sendData(0x07);
	sendData(0x42);
	sendData(0x18);

	sendCom(0xD1);
	sendData(0x00);
	sendData(0x07);
	sendData(0x10);

	sendCom(0xD2);
	sendData(0x01);
	sendData(0x02);

	sendCom(0xC0);
	sendData(0x10);
	sendData(0x3B);
	sendData(0x00);
	sendData(0x02);
	sendData(0x11);

	sendCom(0xC5);
	sendData(0x03);

	sendCom(0xC8);
	sendData(0x00);
	sendData(0x32);
	sendData(0x36);
	sendData(0x45);
	sendData(0x06);
	sendData(0x16);
	sendData(0x37);
	sendData(0x75);
	sendData(0x77);
	sendData(0x54);
	sendData(0x0C);
	sendData(0x00);

 	sendCom(0x36);
	sendData(0xCA);
	X_SIZE = 320;	Y_SIZE = 480;

	sendCom(0x3A);
	sendData(0x55);

	sendCom(0x2A);
	sendData(0x00);
	sendData(0x00);
	sendData(0x01);
	sendData(0x3F);

	sendCom(0x2B);
	sendData(0x00);
	sendData(0x00);
	sendData(0x01);
	sendData(0xE0);
	_delay_ms(120);
	sendCom(0x29);
}
//==============================================================================
void initTFT(void)
{
	DATA_HI_DDR = 0xFF;
	DATA_LO_DDR = 0xFF;

	RS_DDR |= (1<<RS);
	
	WR_DDR  |= (1<<WR)|(1<<RD);	
	WR_PORT |= (1<<WR)|(1<<RD);

//-----------------------   
	RST_DDR  |= (1<<RST);
	RST_PORT &= ~(1<<RST);
	_delay_ms(10);
	RST_PORT |= (1<<RST);
	_delay_ms(120);

	init9481();
}
//==============================================================================

