#define F_CPU 16000000UL

#include <avr/io.h>
#include <util/delay.h>
#include "font.h"

extern unsigned int X_SIZE;
extern unsigned int Y_SIZE;
extern unsigned long XY_LEN;

#define DATA_HI_DDR   	DDRL
#define DATA_HI_PORT  	PORTL

#define RST_DDR			DDRD
#define RST_PORT		PORTD
#define RST			7

#define LCD_C (*(volatile uint8_t*)(0x8000))
#define LCD_D (*(volatile uint8_t*)(0xC000))

extern unsigned int ccc[256];

//==============================================================================
void sendCom(unsigned char cmd)
{	LCD_C = cmd;
}
//==============================================================================
void sendData(unsigned char data)
{	LCD_D = data;
}
//==============================================================================
void setAddrWindow(
	unsigned int x1,unsigned int y1,
	unsigned int x2,unsigned int y2)
{	LCD_C = 0x2A;
	LCD_D = (x1>>8);	LCD_D = x1;
	LCD_D = (x2>>8);	LCD_D = x2;

	LCD_C = 0x2B;
	LCD_D = (y1>>8);	LCD_D = y1;
	LCD_D = (y2>>8);	LCD_D = y2;
}
//==============================================================================
void drawPixel(
	unsigned int x,unsigned int y, //   ?
	unsigned int color)
{  	setAddrWindow(x,y,x,y);
	LCD_C = 0x2C;
	LCD_D = (color>>8);	LCD_D = color;
}
//==============================================================================
void drawChar(
	unsigned int x, unsigned int y,
	unsigned int color, unsigned int backcolor, 
	unsigned char ch)
{	
	unsigned char value = 0;
	unsigned char mask[8] ={128,64,32,16,8,4,2,1};

   	int v,h;
	int tcolor;

	for(h = 0; h < 5; h++)
  	{	value = pgm_read_byte(&(font5x8[ch-32][h]));
		for(v = 0; v < 8; v++)
		{	if ((value & mask[v]) != 0) tcolor = color;
			else tcolor = backcolor;
			drawPixel(x+v,y+h,tcolor);
		}
  	}
}
//==============================================================================
void print_str(
		unsigned int x_pos, unsigned int y_pos,
		unsigned int colour, unsigned int back_colour,
		char *str)
{	while (*str)
	{	drawChar(x_pos,y_pos,colour,back_colour,*str++);
		y_pos += 6;
	}
}
//==============================================================================
//   
// TCNT1 = 966 = 61824 u
void fillArea(unsigned int color, unsigned long len)
{  	
	unsigned int blocks= (unsigned int)(len/16);
  	unsigned char hi = (color>>8), lo = color;

	LCD_C = 0x2C;
	LCD_D = hi; LCD_D = lo;

#define mFOUT LCD_D = hi; LCD_D = lo;
	while(blocks--)
	{	mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			
 
}
//==============================================================================
//   
// TCNT1 = 2765 = 176960 usec
void fillAreaM(unsigned int color, unsigned long len)
{  	unsigned int blocks = (unsigned int)(len/16);
  	unsigned char hi = (color>>8), lo = color;
	unsigned int i = 0;

	sendCom(0x2C);
	LCD_D = hi; LCD_D = lo;

//  
#define mFOUT \
	LCD_D = (ccc[i]>>8); LCD_D = ccc[i];\
	i++; i &= 255; 

	while(blocks--)
	{	mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			

}
//==============================================================================
void fillScreen(unsigned int color)
{  	setAddrWindow(0,0,X_SIZE,Y_SIZE);
  	fillArea(color,XY_LEN);
}
//==============================================================================
void fillScreenM(unsigned int color)
{ 
 	setAddrWindow(0,0,X_SIZE,Y_SIZE);
	fillAreaM(color,XY_LEN);
}
//==============================================================================
void init9488(void)
{
  	sendCom(0x01); // Software Reset

	sendCom(0xC0);  // Power Control 1 [0E 0E]
	sendData(0x10); 
	sendData(0x10);
	
	sendCom(0xC1);	// Power Control 2 [43]
	sendData(0x41);      
	
	sendCom(0xC5); 	// VCOM  Control 1 [00 40 00 40]
	sendData(0x00); 
	sendData(0x22); 
	sendData(0x80); 
	sendData(0x40);
	
 	sendCom(0x36);
	sendData(0x88);
	X_SIZE = 320;	Y_SIZE = 480;

	sendCom(0xB0);	// Interface [00]
	sendData(0x00); 
	
	sendCom(0xB1);	// Frame Rate Control [B0 11]
	sendData(0xB0);
	sendData(0x11);
	
	sendCom(0xB4);	// Inversion Control [02]
	sendData(0x02);
	
	sendCom(0xB6);  // Display Function Control [02 02 3B] .kbv NL=480
	sendData(0x02);
	sendData(0x02);
	sendData(0x3B);
	
	sendCom(0xB7);	// Entry Mode [06]
	sendData(0xC6);
	
	sendCom(0x3A);  // Interlace Pixel Format [XX]
	sendData(0x55);
	
	sendCom(0xF7);	// Adjustment Control 3 [A9 51 2C 82]
	sendData(0xA9); 
	sendData(0x51); 
	sendData(0x2C); 
	sendData(0x82);
	
	sendCom(0x11);		// sleep out

	_delay_ms (120);

	sendCom(0x29);		//  
}
//==============================================================================
void initTFT(void)
{
	DATA_HI_DDR = 0xFF;

//-----------------------   
	RST_DDR  |= (1<<RST);
	RST_PORT &= ~(1<<RST);
	_delay_ms(10);
	RST_PORT |= (1<<RST);
	_delay_ms(120);

	init9488();
}
//==============================================================================
