#define F_CPU 16000000UL

#include <avr/io.h>
#include <util/delay.h>

#include "font.h"

#define DATA_DDR 	DDRA
#define DATA_PORT	PORTA

#define RS_DDR 		DDRC
#define RS_PORT 	PORTC

#define WR_DDR		DDRG	// WR RD
#define WR_PORT		PORTG

#define RST_DDR		DDRD 		// rst
#define RST_PORT	PORTD

#define RST			7	//    ;

#define RS			6	// /,     , 
						//    - ;
#define WR			0	//   ,   ;
#define RD			1	//   ,   ;

extern unsigned int X_SIZE;
extern unsigned int Y_SIZE;
extern unsigned long XY_LEN;

extern unsigned int ccc[256];

//==============================================================================
void sendCom(unsigned char cmd)
{  	RS_PORT &= ~(1<<RS);
  	DATA_PORT = cmd;
  	WR_PORT &= ~(1<<WR);
  	WR_PORT |= (1<<WR);
}
//==============================================================================
void sendData(unsigned char data)
{  	RS_PORT |= (1<<RS);
  	DATA_PORT = data;
  	WR_PORT &= ~(1<<WR);
  	WR_PORT |= (1<<WR);
}
//==============================================================================
void sendByte(unsigned char data)
{  	DATA_PORT = data;
  	WR_PORT &= ~(1<<WR);
  	WR_PORT |= (1<<WR);
}
//==============================================================================
void setAddrWindow(
	unsigned int x1,unsigned int y1,
	unsigned int x2,unsigned int y2)
{	
	sendCom(0x2A);
	sendData(x1>>8);	sendData(x1);
	sendData(x2>>8);	sendData(x2);

	sendCom(0x2B);
	sendData(y1>>8);	sendData(y1);
	sendData(y2>>8);	sendData(y2);
}
//==============================================================================
void drawPixel(int x, int y, unsigned int color)
{	setAddrWindow(x,y,x,y);
	sendCom(0x2C);
  	sendData(color>>8);
	sendByte(color);
}
//==============================================================================
void drawChar(
	unsigned int x, unsigned int y,
	unsigned int color, unsigned int backcolor, 
	unsigned char ch)
{	
	unsigned char value = 0;
	unsigned char mask[8] ={128,64,32,16,8,4,2,1};

   	int v,h;
	int tcolor;

	for(h = 0; h < 5; h++)
  	{	
		value = pgm_read_byte(&(font5x8[ch-32][h]));

		for(v = 0; v < 8; v++)
		{	if ((value & mask[v]) != 0) tcolor = color;
			else tcolor = backcolor;
			drawPixel(x+v,y+h,tcolor);
		}
  	}
}
//==============================================================================
void print_str(
		unsigned int x_pos, unsigned int y_pos,
		unsigned int colour, unsigned int back_colour,
		char *str)
{	while (*str)
	{	drawChar(x_pos,y_pos,colour,back_colour,*str++);
		y_pos += 6;
	}
}
//==============================================================================
//   
// TCNT1 = 4875*64 uS = 312000 uS
void fillArea(unsigned int color, unsigned long len)
{
  	unsigned char hi = (color>>8), lo = color;
	sendCom(0x2C);
	sendData(hi);	sendData(lo);		//   RS

	unsigned int blocks = (unsigned short)(len/16);//64 pixels/block

//  
#define mFOUT sendByte(hi); sendByte(lo);

	while(blocks--)
	{		
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			
}
//==============================================================================
//   
// TCNT1 = 6825 = 436800 usec
void fillAreaM(unsigned int color, unsigned long len)
{
  	unsigned int blocks = (unsigned int)(len/16);
  	unsigned char hi = (color>>8), lo = color;
	unsigned int i = 0;

	sendCom(0x2C);
	sendData(hi);	sendData(lo);

#define mFOUT \
	sendByte(ccc[i]>>8); sendByte(ccc[i]);\
	i++; i &= 255; 

	while(blocks--)
	{
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
		mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT mFOUT
	}
#undef mFOUT			

}
//==============================================================================
void fillScreen(unsigned int color)
{ 	setAddrWindow(0,0,X_SIZE,Y_SIZE);
 	fillArea(color,XY_LEN);
}
//==============================================================================
void fillScreenM(unsigned int color)
{ 	setAddrWindow(0,0,X_SIZE,Y_SIZE);
	fillAreaM(color,XY_LEN);
}
//==============================================================================
void init9488(void)
{
  	sendCom(0x01); // Software Reset

	sendCom(0xC0);  // Power Control 1 [0E 0E]
	sendData(0x10); 
	sendData(0x10);
	
	sendCom(0xC1);	// Power Control 2 [43]
	sendData(0x41);      
	
	sendCom(0xC5); 	// VCOM  Control 1 [00 40 00 40]
	sendData(0x00); 
	sendData(0x22); 
	sendData(0x80); 
	sendData(0x40);
	
 	sendCom(0x36);
	sendData(0x88);
	X_SIZE = 320;	Y_SIZE = 480;

	sendCom(0xB0);	// Interface [00]
	sendData(0x00); 
	
	sendCom(0xB1);	// Frame Rate Control [B0 11]
	sendData(0xB0);
	sendData(0x11);
	
	sendCom(0xB4);	// Inversion Control [02]
	sendData(0x02);
	
	sendCom(0xB6);  // Display Function Control [02 02 3B] .kbv NL=480
	sendData(0x02);
	sendData(0x02);
	sendData(0x3B);
	
	sendCom(0xB7);	// Entry Mode [06]
	sendData(0xC6);
	
	sendCom(0x3A);  // Interlace Pixel Format [XX]
	sendData(0x55);
	
	sendCom(0xF7);	// Adjustment Control 3 [A9 51 2C 82]
	sendData(0xA9); 
	sendData(0x51); 
	sendData(0x2C); 
	sendData(0x82);
	
	sendCom(0x11);		// sleep out

	_delay_ms (120);

	sendCom(0x29);		//  
}
//==============================================================================
void initTFT(void)
{
	DATA_PORT 	= 0x00;
	DATA_DDR  	= 0xFF;			//   

	RS_DDR |= (1<<RS);
	
	WR_DDR  |= (1<<WR)|(1<<RD);
	WR_PORT |= (1<<WR)|(1<<RD);

//-----------------------   
	RST_DDR  |= (1<<RST);
	RST_PORT &= ~(1<<RST);
	_delay_ms(10);
	RST_PORT |= (1<<RST);
	_delay_ms(120);

	init9488();
}
//==============================================================================
